# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2021-03-03 15:13
from __future__ import unicode_literals

from django.db import migrations, models

from dj_dynamic_settings.compat import JSONField
import dj_dynamic_settings.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Setting',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=128, unique=True, validators=[dj_dynamic_settings.models.registry_contains_validator])),
                ('value', JSONField()),
                ('is_active', models.BooleanField(default=True)),
                ('created_date', models.DateTimeField(auto_now_add=True)),
                ('modified_date', models.DateTimeField(auto_now=True)),
            ],
        ),
    ]



from rest_framework.routers import DefaultRouter

from dj_dynamic_settings.views import DynamicSettingsViewSet

router = DefaultRouter()
router.register(r"dynamic_settings", DynamicSettingsViewSet)

urlpatterns = router.urls



SECRET_KEY = "SECRET"

INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "dj_dynamic_settings",
]

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql",
        "NAME": "postgres",
        "USER": "postgres",
        "PASSWORD": "postgres",
        "HOST": "localhost",
        "PORT": "5432",
    }
}

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.dummy.DummyCache",
    }
}



from datetime import datetime

from django.core.exceptions import ValidationError
from django.utils.module_loading import import_string
from rest_framework import serializers
from rest_framework.utils.humanize_datetime import humanize_strptime

from dj_dynamic_settings.metadata import SimpleMetadata


class BaseValidator(object):
    @property
    def detail(self):
        raise NotImplementedError


class TypeValidator(BaseValidator):
    message = "'{value}' is not valid. Valid types are {types}."
    label_lookup = {
        "bool": "boolean",
        "str": "string",
        "unicode": "string",
        "bytes": "string",
        "int": "integer",
        "float": "float",
        "Decimal": "decimal",
        "dict": "nested object",
        "list": "list",
        "newstr": "string",
        "newlist": "list",
        "newint": "int",
        "newbytes": "string",
        "newdict": "nested object",
    }

    def __init__(self, *types):
        assert types, "At least one class has to be provided."
        self.types = types

    def __call__(self, value):
        if not isinstance(value, self.types):
            types = ", ".join(
                self.label_lookup.get(t.__name__, t.__name__) for t in self.types
            )
            raise ValidationError(self.message.format(value=value, types=types))

    @property
    def detail(self):
        return {
            "types": [self.label_lookup.get(t.__name__, t.__name__) for t in self.types]
        }


class ModulePathValidator(BaseValidator):
    message = "{path} not a valid module path."

    def __call__(self, value):
        try:
            import_string(value)
        except ImportError:
            raise ValidationError(self.message.format(path=value))

    @property
    def detail(self):
        return {"types": ["module_path"]}


class DateTimeValidator(BaseValidator):
    message = "DateTime format does not match. DateTime: {value} Format: {dformat}"

    def __init__(self, date_time_format):
        assert date_time_format, "Datetime format has to be provided."
        self.date_time_format = date_time_format

    def __call__(self, value):
        try:
            datetime.strptime(value, self.date_time_format)
        except ValueError:
            date_time_format = humanize_strptime(self.date_time_format)
            raise ValidationError(
                self.message.format(value=value, dformat=date_time_format)
            )

    @property
    def detail(self):
        return {
            "types": ["datetime"],
            "format": humanize_strptime(self.date_time_format),
        }


class SerializerValidator(BaseValidator):
    def __init__(self, serializer_class, serializer_kwargs=None):
        self.serializer_class = serializer_class
        self.serializer_kwargs = serializer_kwargs

    def get_serializer_kwargs(self):
        serializer_kwargs = self.serializer_kwargs or {}
        if callable(serializer_kwargs):
            serializer_kwargs = serializer_kwargs()
        assert "data" not in serializer_kwargs
        assert "instance" not in serializer_kwargs
        return serializer_kwargs

    def __call__(self, value):
        serializer = self.serializer_class(data=value, **self.get_serializer_kwargs())
        serializer.is_valid(raise_exception=True)

    @property
    def detail(self):
        meta = SimpleMetadata()
        serializer = self.serializer_class(**self.get_serializer_kwargs())
        params = {}
        if isinstance(serializer, serializers.ListSerializer):
            params["types"] = [TypeValidator.label_lookup.get("list")]
            if isinstance(serializer.child, serializers.Serializer):
                params["fields"] = meta.get_serializer_info(serializer)
            else:
                params["field"] = meta.get_field_info(serializer.child)
        else:
            params["types"] = [TypeValidator.label_lookup.get("dict")]
            params["fields"] = meta.get_serializer_info(serializer)
        return params


class FieldValidator(BaseValidator):
    def __init__(self, field_class, field_kwargs=None):
        self.field_class = field_class
        self.field_kwargs = field_kwargs

    def get_field_kwargs(self):
        field_kwargs = self.field_kwargs or {}
        if callable(field_kwargs):
            field_kwargs = field_kwargs()
        return field_kwargs

    def __call__(self, value):
        self.field_class(**self.get_field_kwargs()).run_validation(value)

    @property
    def detail(self):
        meta = SimpleMetadata()
        field = self.field_class(**self.get_field_kwargs())
        return {"types": ["field"], "field_info": meta.get_field_info(field)}



from rest_framework import authentication, permissions, viewsets
from rest_framework.response import Response

from dj_dynamic_settings.compat import action
from dj_dynamic_settings.models import Setting
from dj_dynamic_settings.registry import registry
from dj_dynamic_settings.serializers import (
    DynamicSettingSerializer,
    RegistryItemSerializer,
)


class DynamicSettingsViewSet(viewsets.ModelViewSet):
    queryset = Setting.objects.all()
    serializer_class = DynamicSettingSerializer
    permission_classes = (permissions.IsAdminUser,)
    authentication_classes = (
        authentication.SessionAuthentication,
        authentication.TokenAuthentication,
    )

    @action(detail=False, methods=["GET"], url_path="registry")
    def get_registry(self, *args, **kwargs):
        items = [registry[key] for key in registry.keys()]
        serializer = RegistryItemSerializer(items, many=True)
        return Response(serializer.data)



from django.test import TestCase

from dj_dynamic_settings.conf import settings
from dj_dynamic_settings.registry import Unset, registry
from dj_dynamic_settings.utils import override_settings


class OverrideSettingsTestCase(TestCase):
    def setUp(self):
        from dj_dynamic_settings.tests import definitions  # NoQA

        # in order to register test settings

    def test_setting_default_value(self):
        self.assertIsNotNone(settings.X_FEATURE_CONFIGURATION_WITH_DEFAULT_VALUE)
        self.assertNotEqual(settings.X_FEATURE_CONFIGURATION_WITH_DEFAULT_VALUE, Unset)

    def test_setting_with_statement(self):
        default_value = settings.X_FEATURE_CONFIGURATION_WITH_DEFAULT_VALUE
        new_value = "override_settings"

        with override_settings(X_FEATURE_CONFIGURATION_WITH_DEFAULT_VALUE=new_value):
            self.assertEqual(
                settings.X_FEATURE_CONFIGURATION_WITH_DEFAULT_VALUE, new_value
            )

        self.assertEqual(
            settings.X_FEATURE_CONFIGURATION_WITH_DEFAULT_VALUE, default_value
        )

    @override_settings(X_FEATURE_CONFIGURATION_WITH_DEFAULT_VALUE="override_settings")
    def test_setting_with_decoration(self):
        self.assertEqual(
            settings.X_FEATURE_CONFIGURATION_WITH_DEFAULT_VALUE, "override_settings"
        )

    def test_setting_without_default_value(self):
        with self.assertRaises(AttributeError):
            settings.X_TRIAL_COUNT

        self.assertTrue("X_TRIAL_COUNT" in registry)

        with override_settings(X_TRIAL_COUNT=3):
            self.assertEqual(3, settings.X_TRIAL_COUNT)


@override_settings(X_TRIAL_COUNT=3)
class OverrideSettingsWithDecoratorTestCase(TestCase):
    def setUp(self):
        from dj_dynamic_settings.tests import definitions  # NoQA

        # in order to register test settings

    def test_decorator(self):
        self.assertEqual(settings.X_TRIAL_COUNT, 3)

    def test_decorator_with_nested(self):
        with override_settings(X_TRIAL_COUNT=4, X_FEATURE_ACTIVE=True):
            with override_settings(X_TRIAL_COUNT=5):
                self.assertEqual(settings.X_TRIAL_COUNT, 5)
                self.assertTrue(settings.X_FEATURE_ACTIVE)

            self.assertEqual(settings.X_TRIAL_COUNT, 4)
            self.assertTrue(settings.X_FEATURE_ACTIVE)

        self.assertEqual(settings.X_TRIAL_COUNT, 3)



import django

__all__ = ["action", "JSONField"]


try:
    from rest_framework.decorators import action
except ImportError:
    from rest_framework.decorators import detail_route, list_route

    def action(methods=None, detail=None, url_path=None, **kwargs):
        assert detail is not None, "@action() missing required argument: 'detail'"
        route = detail and detail_route or list_route
        return route(methods=methods, url_path=url_path, **kwargs)


if django.VERSION >= (3, 1):
    from django.db.models import JSONField
else:
    from django.contrib.postgres.fields import JSONField



from django.test.utils import TestContextDecorator

from dj_dynamic_settings.conf import settings


class OverrideSettings(TestContextDecorator):
    def __init__(self, **kwargs):
        self.options = kwargs
        super(OverrideSettings, self).__init__()
        self.previous = None

    def enable(self):
        self.previous = settings.holder
        new_holder = dict(**self.previous) if self.previous else {}
        new_holder.update(**self.options)
        settings.holder = new_holder

    def disable(self):
        settings.holder = self.previous
        self.previous = None


override_settings = OverrideSettings



from __future__ import unicode_literals

from django.core.exceptions import ValidationError
from django.db import models
from django.db.models.signals import post_delete, post_save

from dj_dynamic_settings.app_settings import app_settings
from dj_dynamic_settings.compat import JSONField
from dj_dynamic_settings.conf import delete_settings_cache, refresh_settings_cache
from dj_dynamic_settings.registry import registry


def registry_contains_validator(value):
    if value not in registry.keys():
        raise ValidationError('"%s" is not a valid choice.' % value)


class Setting(models.Model):
    key = models.CharField(
        max_length=128,
        validators=[registry_contains_validator],
        unique=True,
    )
    value = JSONField()
    is_active = models.BooleanField(default=True)
    created_date = models.DateTimeField(auto_now_add=True)
    modified_date = models.DateTimeField(auto_now=True)

    def __str__(self):
        return "%s = %s" % (self.key, self.value)


if app_settings.USE_CACHE:
    post_save.connect(refresh_settings_cache, sender=Setting)
    post_delete.connect(delete_settings_cache, sender=Setting)



from rest_framework import serializers

from dj_dynamic_settings.models import Setting
from dj_dynamic_settings.registry import Unset, registry


class DynamicSettingSerializer(serializers.ModelSerializer):
    default = serializers.SerializerMethodField(method_name="get_default_")
    description = serializers.SerializerMethodField()

    def get_default_(self, instance):
        setting = registry[instance.key]
        default = setting.default
        if default is Unset:
            default = default.__name__
        return default

    def get_description(self, instance):
        setting = registry[instance.key]
        return setting.description

    def validate(self, attrs):
        if self.instance:
            key = attrs.get("key", self.instance.key)
            value = attrs.get("value", self.instance.value)
        else:
            key = attrs["key"]
            value = attrs["value"]
        validators = registry[key].validators
        for validator in validators:
            validator(value)
        return attrs

    class Meta:
        model = Setting
        fields = [
            "pk",
            "key",
            "value",
            "default",
            "is_active",
            "description",
            "created_date",
            "modified_date",
        ]
        extra_kwargs = {
            "created_date": {"read_only": True},
            "modified_date": {"read_only": True},
        }


class RegistryItemSerializer(serializers.Serializer):
    key = serializers.CharField()
    description = serializers.CharField()
    default = serializers.SerializerMethodField(method_name="get_default_")
    validators = serializers.SerializerMethodField(method_name="get_validators_")

    def get_default_(self, instance):  # noqa
        default = instance.default
        if default is Unset:
            default = default.__name__
        return default

    def get_validators_(self, instance):  # noqa
        return [
            validator.detail
            for validator in instance.validators
            if hasattr(validator, "detail")
        ]



from django.conf import settings as django_settings
from django.core.cache import DEFAULT_CACHE_ALIAS

__all__ = ["app_settings"]


defaults = {
    "CACHE_KEY_PREFIX": "dynamic_settings",
    "CACHE_ALIAS": DEFAULT_CACHE_ALIAS,
    "USE_CACHE": True,
    "CACHE_TIMEOUT": 1 * 60 * 60,
}


class AppSettings(object):
    def __init__(self, prefix):
        for key, value in defaults.items():
            dj_setting_key = "%s%s" % (prefix, key)
            self.__setattr__(key, getattr(django_settings, dj_setting_key, value))


app_settings = AppSettings("DYNAMIC_SETTINGS_")



from django.db import models
from rest_framework.fields import empty
from rest_framework.metadata import SimpleMetadata as _SimpleMetadata

from dj_dynamic_settings.registry import Unset


class SimpleMetadata(_SimpleMetadata):
    def get_field_info(self, field):
        field_info = super(SimpleMetadata, self).get_field_info(field)
        if hasattr(field, "default"):
            default = field.default
            if default is empty:
                default = Unset.__name__
            if callable(default):
                default = default()
            if isinstance(default, models.Model):
                default = default.pk
            field_info["default"] = default

        if "input_type" in field.style:
            field_info["type"] = field.style["input_type"]
        return field_info



from rest_framework import serializers

from dj_dynamic_settings.registry import BaseSetting, registry
from dj_dynamic_settings.validators import (
    DateTimeValidator,
    ModulePathValidator,
    SerializerValidator,
    TypeValidator,
)


class FeatureConfigurationSerializer(serializers.Serializer):
    value_a = serializers.BooleanField()
    value_b = serializers.FloatField()


class FeatureRuleSerializer(serializers.Serializer):
    value_c = serializers.BooleanField()
    value_d = serializers.FloatField()
    value_e = serializers.CharField(validators=[ModulePathValidator()])


@registry.register
class FeatureActive(BaseSetting):
    key = "X_FEATURE_ACTIVE"
    validators = [TypeValidator(bool)]


@registry.register
class DefaultFeatureActive(BaseSetting):
    key = "Y_FEATURE_ACTIVE"
    validators = [TypeValidator(bool)]
    default = True


@registry.register
class TrialCount(BaseSetting):
    key = "X_TRIAL_COUNT"
    validators = [TypeValidator(int)]


@registry.register
class ServiceClass(BaseSetting):
    key = "X_SERVICE_CLASS"
    validators = [ModulePathValidator()]


@registry.register
class FeatureConfiguration(BaseSetting):
    key = "X_FEATURE_CONFIGURATION"
    validators = [SerializerValidator(FeatureConfigurationSerializer)]


@registry.register
class FeatureRulesConfiguration(BaseSetting):
    key = "X_FEATURE_RULES"
    validators = [
        SerializerValidator(FeatureRuleSerializer, serializer_kwargs={"many": True})
    ]


@registry.register
class FeatureConfigurationWithDefaultValue(BaseSetting):
    key = "X_FEATURE_CONFIGURATION_WITH_DEFAULT_VALUE"
    validators = [TypeValidator(str)]
    default = "default_value"


@registry.register
class FeatureDateTimeConfiguration(BaseSetting):
    key = "X_DATETIME_CONFIGURATION"
    validators = [DateTimeValidator("%d-%m-%Y")]



from django.test import TestCase

from dj_dynamic_settings.conf import settings
from dj_dynamic_settings.models import Setting
from dj_dynamic_settings.registry import registry
from dj_dynamic_settings.serializers import (
    DynamicSettingSerializer,
    RegistryItemSerializer,
)


class DynamicSettingsTestCase(TestCase):
    def setUp(self):
        from dj_dynamic_settings.tests import definitions

        # in order to register test settings

    def test_existing_and_active_setting(self):
        Setting.objects.create(
            key="X_FEATURE_ACTIVE",
            value=True,
            is_active=True,
        )
        self.assertTrue(settings.X_FEATURE_ACTIVE)

    def test_existing_and_inactive_setting(self):
        Setting.objects.create(
            key="X_FEATURE_ACTIVE",
            value=True,
            is_active=False,
        )
        with self.assertRaises(AttributeError):
            _ = settings.X_FEATURE_ACTIVE

    def test_default_setting(self):
        self.assertEqual(settings.Y_FEATURE_ACTIVE, True)

    def test_non_existing_setting(self):
        with self.assertRaises(AttributeError):
            _ = settings.NON_EXISTING_SETTING


class DynamicSettingsValidationTestCase(TestCase):
    def setUp(self):
        from dj_dynamic_settings.tests import definitions

        # in order to register test settings
        self.serializer_class = DynamicSettingSerializer

    def test_type_validated_setting(self):
        data = {
            "key": "X_FEATURE_ACTIVE",
            "value": 1,
            "is_active": True,
        }
        serializer = self.serializer_class(data=data)
        self.assertFalse(serializer.is_valid())

        data["value"] = True
        serializer = self.serializer_class(data=data)
        self.assertTrue(serializer.is_valid())

        data = {
            "key": "X_TRIAL_COUNT",
            "value": "abc",
            "is_active": True,
        }
        serializer = self.serializer_class(data=data)
        self.assertFalse(serializer.is_valid())

        data["value"] = 1
        serializer = self.serializer_class(data=data)
        self.assertTrue(serializer.is_valid())

    def test_module_path_validated_setting(self):
        data = {
            "key": "X_SERVICE_CLASS",
            "value": "django.test.TestCase",
            "is_active": True,
        }
        serializer = self.serializer_class(data=data)
        self.assertTrue(serializer.is_valid())

        data["value"] = "invalid.module.path"
        serializer = self.serializer_class(data=data)
        self.assertFalse(serializer.is_valid())

    def test_serializer_validated_setting(self):
        data = {
            "key": "X_FEATURE_CONFIGURATION",
            "value": {"value_a": True, "value_b": 1.2},
            "is_active": True,
        }
        serializer = self.serializer_class(data=data)
        self.assertTrue(serializer.is_valid())

        data["value"] = {"value_a": "abc", "value_b": "cde"}
        serializer = self.serializer_class(data=data)
        self.assertFalse(serializer.is_valid())

        data = {
            "key": "X_FEATURE_RULES",
            "value": [
                {"value_c": True, "value_d": 6.1, "value_e": "django.db.models"},
                {
                    "value_c": False,
                    "value_d": 4.9,
                    "value_e": "django.core.validators.RegexValidator",
                },
            ],
            "is_active": True,
        }
        serializer = self.serializer_class(data=data)
        self.assertTrue(serializer.is_valid())

    def test_datetime_validated_setting(self):
        data = {
            "key": "X_DATETIME_CONFIGURATION",
            "value": "01/04/2017",
            "is_active": True,
        }
        serializer = self.serializer_class(data=data)
        self.assertFalse(serializer.is_valid())

        data["value"] = "01-04-2017"

        serializer = self.serializer_class(data=data)
        self.assertTrue(serializer.is_valid())

    def test_validator_details(self):
        items = [registry[key] for key in registry.keys()]
        serializer = RegistryItemSerializer(items, many=True)
        data = serializer.data
        self.assertGreater(len(data), 0)
        for item in data:
            self.assertTrue(item.get("validators"))



from django.apps import apps
from django.core.cache import caches
from django.db import transaction
from django.utils.functional import cached_property

from dj_dynamic_settings.app_settings import app_settings
from dj_dynamic_settings.exceptions import SettingNotFoundException
from dj_dynamic_settings.registry import Unset, registry


class Settings(object):
    holder = None

    def __getattr__(self, key):
        if self.holder and (key in self.holder):
            return self.holder[key]

        try:
            return self.get_value(key)
        except SettingNotFoundException:
            raise AttributeError(
                "%s object has no attribute: %s" % (self.__class__.__name__, key)
            )

    def get_value(self, key, check_cache=True):
        if key not in registry:
            raise SettingNotFoundException()

        if app_settings.USE_CACHE and check_cache:
            value = self._get_from_cache(key)
            if not (value is Unset):
                return value

        value = self._get_value(key)
        self._write_to_cache(key, value)
        return value

    def _get_from_cache(self, key):
        cache_key = self._get_cache_key(key)
        value = self.cache.get(cache_key, Unset)
        return value

    def _write_to_cache(self, key, value):
        if not app_settings.USE_CACHE:
            return
        cache_key = self._get_cache_key(key)
        self.cache.set(cache_key, value, timeout=app_settings.CACHE_TIMEOUT)

    def _delete_from_cache(self, key):
        cache_key = self._get_cache_key(key)
        self.cache.delete(cache_key)

    @staticmethod
    def _get_default_value(key):
        setting = registry[key]
        if not (setting.default is Unset):
            return setting.default
        raise SettingNotFoundException()

    def _get_value(self, key):
        if not apps.is_installed("dj_dynamic_settings"):
            return self._get_default_value(key)

        from dj_dynamic_settings.models import Setting

        try:
            setting = Setting.objects.get(key=key, is_active=True)
            return setting.value
        except Setting.DoesNotExist:
            return self._get_default_value(key)

    @cached_property
    def cache(self):
        return caches[app_settings.CACHE_ALIAS]

    @staticmethod
    def _get_cache_key(key):
        return "%s__%s" % (app_settings.CACHE_KEY_PREFIX, key)

    def update(self, setting):
        if setting.is_active:
            self._write_to_cache(setting.key, setting.value)
        else:
            self._delete_from_cache(setting.key)

    def delete(self, key):
        self._delete_from_cache(key)


settings = Settings()


def refresh_settings_cache(sender, instance, *args, **kwargs):
    transaction.on_commit(lambda: settings.update(instance))


def delete_settings_cache(sender, instance, *args, **kwargs):
    transaction.on_commit(lambda: settings.delete(instance.key))



#!/usr/bin/env python
import os
import sys
from setuptools import find_packages, setup

sys.path.insert(0, os.path.dirname(__file__))

try:
    from version import __version__
except ImportError:
    __version__ = '0.0.1'

__author__ = 'Akinon'
__license__ = 'MIT'
__maintainer__ = 'Akinon'
__email__ = 'dev@akinon.com'

if sys.version_info[0] == 2:
    from io import open

with open('README.md', 'r', encoding='utf-8') as readme:
    long_description = readme.read()

setup(
    name='dj-dynamic-settings',
    version=__version__,
    author=__author__,
    author_email=__email__,
    maintainer=__maintainer__,
    maintainer_email=__email__,
    description='Stay informed of it',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license=__license__,
    url='https://bitbucket.org/akinonteam/dj-dynamic-settings/',
    project_urls={
        'Documentation': 'https://bitbucket.org/akinonteam/dj-dynamic-settings/',
        'Source Code': 'https://bitbucket.org/akinonteam/dj-dynamic-settings/',
    },
    packages=find_packages(exclude=['*.tests', '*.tests.*', 'tests.*', 'tests']),
    platforms='any',
    zip_safe=False,
    use_scm_version={
        'write_to': './version.py',
        'write_to_template': '__version__ = "{version}"\n',
    },
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Framework :: Django",
        "Framework :: Django :: 1.10",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
    ],
)



from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.core.validators import RegexValidator

setting_key_regex = r"^[A-Z]{1}[A-Z0-9_]*$"
setting_key_validator = RegexValidator(regex=setting_key_regex)


class Unset(object):
    pass


class BaseSetting(object):
    key = None
    validators = []
    default = Unset
    description = None


class SettingsRegistry(object):
    def __init__(self):
        self._registry = {}

    def register(self, setting_cls):
        if setting_cls.key in self._registry:
            raise ValueError("%s already registered." % setting_cls.key)

        try:
            setting_key_validator(setting_cls.key)
        except ValidationError:
            raise ImproperlyConfigured(
                "'%s' does not match regex '%s'." % (setting_cls.key, setting_key_regex)
            )

        self._registry[setting_cls.key] = setting_cls

    def keys(self):
        return self._registry.keys()

    def __getitem__(self, item):
        return self._registry[item]

    def __contains__(self, item):
        return self._registry.__contains__(item)


registry = SettingsRegistry()
